'
' BSODomizer
'
' By Kingpin and Zoz
' March-July 2008
'
' v1.1 updated November 2008 by Kingpin
' Fixed HSYNC detection issue, now supports both active-high and active-low HSYNC pulses
'
' v1.2 updated 2011-04-07 by 501d3r guy
' Reworked IR reader to allow system clock frequency independence as well
' as operation down to 10 MHz (xtal1 + plll2x)


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  ButtonPin = 0
  Dip1Pin = 7
  Dip2Pin = 6
  Dip3Pin = 5
  Dip4Pin = 4
  VideoActivityPin = 10
  VideoSelectPin = 15
  IRRemotePin = 24

  cols = vgatext#cols
  rows = vgatext#rows
  chrs = cols * rows

  shorttimeout = 300
  mediumtimeout = 600
  longtimeout = 1800
  
VAR

  'sync long - written to -1 by VGA driver after each screen refresh
  long  sync
  'screen buffer - could be bytes, but longs allow more efficient scrolling
  long  screen[cols*rows/4]
  'row colors
  word  colors[rows]
  'cursor control bytes
  byte  cx0, cy0, cm0, cx1, cy1, cm1
  'remote control code
  byte remotecode
  'cog for IR remote reading
  byte ircog
  ' boolean to let us know if display is active       
  byte displayactive
  ' keep track of run time in seconds
  word timesecs
  long lastcnt
  ' auto BSOD timeout (selected by dip switches)
  word timeout
  ' state of HSYNC
  byte hsync
  word hsync_timer

OBJ

  clock : "Clock"              ' available from "Clock" object created by Jeff Martin/Parallax, http://obex.parallax.com/objects/139/
  vgatext : "VGA_HiRes_Text"   ' available from "VGA HiRes Text" object created by Chip Gracey/Parallax, http://obex.parallax.com/objects/69/
  remote : "ir_reader_sony"    ' available from "IR Kit" object created by Bob Belleville, http://obex.parallax.com/objects/37/
  
PUB start | i, j
  ' set button and dip switch pins to inputs
  dira[ButtonPin] := 0
  dira[Dip1Pin] := 0
  dira[Dip2Pin] := 0
  dira[Dip3Pin] := 0
  dira[Dip4Pin] := 0
  ' set video signal monitoring pin to input
  dira[VideoActivityPin] := 0
  ' set IR remote control monitoring pin to input
  dira[IRRemotePin] := 0
  ' set video select pin to output
  dira[VideoSelectPin] := 1
  ' set video select to HIGH (regular VGA passthrough) for normal operation
  outa[VideoSelectPin] := 1
  ' display starts off inactive
  displayactive := false

  ' start clock in our initial fast operation
  clock.Init(_xinfreq)
  
  ' normal operation is to slow down to minimum speed to save power
  ' while waiting for either IR trigger signal or auto BSOD timeout
  ' (depending on DIP switch settings)
  ' NOTE: unfortunately, the IR reader cannot operate below 10 MHz, and thus
  ' we must set our min system clock to at least 10 MHz (cannot run in xtal1 + pll1 mode)
  ' As as benefit, however, it can operate while our main cog is in a 'wait' mode,
  ' thus eliminating the IR remote "multiple try" activation problem
  clock.SetClock(xtal1 + pll2x)

  ' start IR remote control reader on its cog
  remote.init(IRRemotePin,0,6,false)   'startup
  ircog := 1

  ' initialize second timing setup
  timesecs := 0
  lastcnt := cnt

  ' if remote control reader started, loop forever in main program loop 
  if ircog > 0
    repeat
      clock.PauseMSec(500)  ' only do this loop at 2 Hz to save power
      ' if we're already displaying a BSOD...
      if displayactive
        ' check buttons and DIP switches
        if ina[ButtonPin] == 0
          repeat
          while ina[ButtonPin] == 0  ' debounce
          outa[VideoSelectPin] := 1
          DeactivateDisplay
        remotecode := remote.fifo_get
        if (remotecode & $7F) == %10000  ' channel up code
          outa[VideoSelectPin] := 1  ' go back to video passthrough mode - this is really a debug option as it will confuse the recipient
          DeactivateDisplay
        if (remotecode & $7F) == %10001  ' channel down code
          ' throw up a BGOD (blue goatse of death) over the BSOD 
          DisplayGoatse

        ' check HSYNC line - the polarity could be active high or active low, so we just have to look for a toggle
        ' if we don't see one for about half a second, we can assume the computer has been turned off
        hsync_timer := 0
        hsync := ina[VideoActivityPin]
        repeat while ina[VideoActivityPin] == hsync
          hsync_timer++
          if hsync_timer > 32767    ' the system has been reset, so go back to video passthrough mode
            outa[VideoSelectPin] := 1
            DeactivateDisplay
            quit
      else
        ' otherwise check for received IR remote code
        ' NOTE: we no longer need to ramp up the clock frequency, as the IR
        ' reader can operation all the way down to xtal1 + pll2x
        clock.PauseMSec(50)  ' give the remote control cog time to get something
        remotecode := remote.fifo_get
        ' NOTE: we no longer need to ramp down the clock frequency, either
        if (remotecode & $7F) == %10101  ' power on/off code
            outa[VideoSelectPin] := 0      ' set video select to LOW (enable propeller BSOD video output)
            ActivateDisplay
            DisplayHeader                  ' Display the text we love so much

            ' if goatse mode is selected, wait some time and throw up the BGOD (blue goatse of death)
            if ina[Dip4Pin] == 0           
              clock.MarkSync
              clock.WaitSyncSec(10)
              bytefill(@screen,$20,chrs)
              DisplayGoatse

        else
          if ina[ButtonPin] == 0  ' button press causes instant BSOD
            outa[VideoSelectPin] := 0      ' set video select to LOW (enable propeller BSOD video output)
            ActivateDisplay
            DisplayHeader                  ' Display the text we love so much
            repeat
            while ina[ButtonPin] == 0      ' debounce

            ' if goatse mode is selected, wait some time and throw up the BGOD (blue goatse of death)
            if ina[Dip4Pin] == 0           
              clock.MarkSync
              clock.WaitSyncSec(10)
              bytefill(@screen,$20,chrs)
              DisplayGoatse
          else  ' check dip switch settings to see if we should throw a timed BSOD
            if (ina[Dip2Pin] == 0) or (ina[Dip3Pin] == 0)  ' timed mode active
              if (cnt - lastcnt) > clkfreq
                timesecs := timesecs + 1
                lastcnt := lastcnt + clkfreq
              else
                if (lastcnt > cnt) and ((cnt + (POSX - lastcnt)) > clkfreq) ' this is a hack that is not completely accurate but WHO CARES
                    timesecs := timesecs + 1
                    lastcnt := cnt
              ' set timeout based on dip switch settings
              if ina[Dip2Pin] == 1
                if ina[Dip3Pin] == 0
                  timeout := shorttimeout
              else
                if ina[Dip3Pin] == 1
                  timeout := mediumtimeout
                else
                  timeout := longtimeout
              if timesecs > timeout
                outa[VideoSelectPin] := 0      ' set video select to LOW (enable propeller BSOD video output)
                ActivateDisplay
                DisplayHeader                  ' Display the text we love so much

                ' if goatse mode is selected, wait some time and throw up the BGOD (blue goatse of death)
                if ina[Dip4Pin] == 0           
                  clock.MarkSync
                  clock.WaitSyncSec(10)
                  bytefill(@screen,$20,chrs)
                  DisplayGoatse
  
PRI ActivateDisplay | i
'' Speed system up to full 80MHz speed and activate display routines
    ' ramp up clock speed
    clock.Init(_xinfreq)
    clock.SetClock(xtal1 + pll16x)
    'start vga text driver
    vgatext.start(16, @screen, @colors, @cx0, @sync)
     
    'set up colors for all the rows
    repeat i from 0 to rows - 1
      if ina[Dip1Pin] == 1  ' Windows mode
        colors[i] := %%0020_3330 ' white text on blue background
      else                  ' Mac mode
        colors[i] := %%0000_3330 ' white text on black background
     
    bytefill(@screen,$20,chrs)    'Clear Screen

    displayactive := true    

PRI DeactivateDisplay
'' Turn off display cogs and slow processor down to minimum speed to save power
    vgatext.stop
    clock.SetClock(xtal1 + pll2x)
    displayactive := false
    timesecs := 0
    lastcnt := cnt    

PRI DisplayHeader
    if ina[Dip1Pin] == 1  ' Windows mode
      DisplayBSOD
    else                  ' Mac mode
      DisplayKernelPanic

PRI DisplayBSOD
    Text(0,1,string("A problem has been detected and Windows has been shut down to prevent damage"))
    Text(0,2,string("to your computer."))
    Text(0,4,string("The problem seems to be caused by the following file: SPCMDCON.SYS"))
    Text(0,6,string("PAGE_FAULT_IN_NONPAGED_AREA"))
    Text(0,8,string("If this is the first time you've seen this Stop error screen,"))
    Text(0,9,string("restart your computer. If this screen appears again, follow"))
    Text(0,10,string("these steps:"))
    Text(0,12,string("Check to make sure any new hardware or software is properly installed."))
    Text(0,13,string("If this is a new installation, ask your hardware or software manufacturer"))
    Text(0,14,string("for any Windows updates you might need."))
    Text(0,16,string("If problems continue, disable or remove any newly installed hardware"))
    Text(0,17,string("or software. Disable BIOS memory options such as caching or shadowing."))
    Text(0,18,string("If you need to use Safe Mode to remove or disable components, restart"))
    Text(0,19,string("your computer, press F8 to select Advanced Startup Options, and then"))
    Text(0,20,string("select Safe Mode."))
    Text(0,22,string("Technical information:"))
    Text(0,24,string("*** STOP: 0x00000050 (0xFD3094C2,0x00000001,0xFBFE7617,0x00000000)"))
    Text(0,27,string("*** SPCMDCON.SYS - Address FBFE7617 base at FBFE5000, DateStamp 3d6dd67c"))
    Text(0,30,string("Beginning dump of physical memory"))
    Text(0,31,string("Physical memory dump complete."))
    Text(0,32,string("Contact your system administrator or technical support group for further"))
    Text(0,33,string("assistance."))

PRI DisplayKernelPanic
    Text(0, 1, string("panic(cpu 0): Couldn't register to modules"))
    Text(0, 3, string("backtrace: 0x000894c0 0x0002doe8 0x000dbe68 0x000dc03c 0x88188fd3 0x0003d264 0x0003b404 0x0003b330"))
    Text(0, 5, string("No debugger configured - dumping debug information"))
    Text(0, 7, string("version string : Darwin Kernel Version 8.11.1:"))
    Text(0, 8, string("Wed Oct 10 18:23:28 PDT 2007; root:xnu-792.25.20~1/RELEASE_I386"))
    Text(0, 11, string("DBAT0: 00000000 00000000"))
    Text(0, 12, string("DBAT1: 00000000 00000000"))
    Text(0, 13, string("DBAT2: 80001FFE 8000003A"))
    Text(0, 14, string("DBAT3: 90001FFE 9000003A"))
    Text(0, 15, string("MSR=00001030"))
    Text(0, 16, string("backtrace: 0x000895ec 0x0002dee8 0x000dbe68 0x000dc03c 0x00188fd0 0x0003d264 0x0003b404 0x0003b338"))
    Text(0, 17, string("panic: We are hanging here..."))                                                                        
    Text(0, 18, string("IOATAPIDVDDrive: Using DMA transfers"))
    Text(0, 19, string("IOUSBHub: USB Generic Hub @ 2"))
    Text(0, 20, string("IOUSBHub: Hub attached - Bus powered"))
    Text(0, 21, string("IOUSBHub: USB Generic Hub @ 3"))
    Text(0, 22, string("IOUSBHub: Hub attached - Self/Bus powered, power supply good"))
    Text(0, 23, string("AppleKeyboard: USB Generic Keyboard @ 4"))
    Text(0, 24, string("AppleMouse: USB Generic Mouse @ 6"))
    Text(0, 25, string("IOSATAHDDRIVE: Using U-DMA/133 transfers"))
    Text(0, 26, string("IOSATAHDDrive: Using U-DMA/133 transfers"))

PRI DisplayGoatse
    Text(128, 18, string("* y o u * h a v e * b e e n * B S O D o m i z e d *"))
    Text(128, 19, string("y                                                 y"))  
    Text(128, 20, string("o  /     \             \            /    \        o"))
    Text(128, 21, string("u |       |             \          |      |       u"))
    Text(128, 22, string("* |       `.             |         |       :      *"))
    Text(128, 23, string("h `        |             |        \|       |      h"))
    Text(128, 24, string("a  \       | /       /  \\\   --__ \\       :     a"))
    Text(128, 25, string("v   \      \/   _--~~          ~--__| \     |     v"))  
    Text(128, 26, string("e    \      \_-~                    ~-_\    |     e"))
    Text(128, 27, string("*     \_     \        _.--------.______\|   |     *"))
    Text(128, 28, string("b       \     \______// _ ___ _ (_(__>  \   |     b"))
    Text(128, 29, string("e        \   .  C ___)  ______ (_(____>  |  /     e"))
    Text(128, 30, string("e        /\ |   C ____)/      \ (_____>  |_/      e"))
    Text(128, 31, string("n       / /\|   C_____)       |  (___>   /  \     n"))
    Text(128, 32, string("*      |   (   _C_____)\______/  // _/ /     \    *"))
    Text(128, 33, string("B      |    \  |__   \\_________// (__/       |   B"))
    Text(128, 34, string("S     | \    \____)   `----   --'             |   S"))
    Text(128, 35, string("O     |  \_          ___\       /_          _/ |  O"))
    Text(128, 36, string("D    |              /    |     |  \            |  D"))
    Text(128, 37, string("o    |             |    /       \  \           |  o"))
    Text(128, 38, string("m    |          / /    |         |  \           | m"))
    Text(128, 39, string("i    |         / /      \__/\___/    |          | i"))
    Text(128, 40, string("z   |           /        |    |       |         | z"))
    Text(128, 41, string("e   |          |         |    |       |         | e"))
    Text(128, 42, string("d   |          |         |    |       |         | d"))
    Text(128, 43, string("* y o u * h a v e * b e e n * B S O D o m i z e d *"))
    
PRI Text(x,y,str)|len,Offset                            'Place text string at location x,y
''      Note: a value of x greater than 'cols'(127) will result in centering the text
    len := strsize(str)                                 'Get string length
    if x > cols -1                                      'if x position > cols then center text
       x := (cols-len)>>1
    Offset := x+y*cols <# cols*rows-len                 'calculate offset
    bytemove(@screen+Offset, str,len)                   'move string to proper location
